/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com> 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Collections;
using System.Text.RegularExpressions;
using System.IO;
using System.Threading;

namespace MobiServer.Devices.IR.Control
{
    public partial class Panel : MobiServer.Devices.ControlBase
    {
        #region Constants/Variables

        Hashtable controlHolder = null;
        Control.Base targetControl = null;
        Hashtable productHolder = null;
       
        int targetHeight = 0;

        const string SETTING_FILE = @"setting_ir.txt";
        const string SETTING_FILE_CATEGORY = "setting_category.txt";
        const string SETTING_FILE_PRODUCT_HEADER = "setting_product";

        string[] TestCommands = { "^(Power|On)$", "^(Play|Enter|OK)$", "^(.+)$" };


        public MobiServer.MessageHandler messageHandler;
        public MobiServer.WriteHandler serverWriteHandler;
        public MobiServer.ReadHandler serverReadHandler;
        string receivedStr = "";

        const int READ_TIMEOUT = 5000;
        const int WRITE_TIMEOUT = 1000;
        
        const string STR_RECEIVE_START = "ԊOM Jn";
        const string STR_RECEIVE_INIT = "ԊOM ҋ@";
        const string STR_RECEIVE_ERROR = "ԊOM s";
        const string STR_RECEIVE_COMPLETE = "ԊOM ";

        const string STR_OPEN_ERROR = "VA|[gJĂ܂";

        const string STR_SEND_START = "ԊOM Jn";
        const string STR_SEND_PORT = "ԊOM |[g";
        const string STR_SEND_INIT = "ԊOM ҋ@";
        const string STR_SEND_ERROR = "ԊOM s";
        const string STR_SEND_COMPLETE = "ԊOM ";

        const string STR_SEND_LENGTH_ERROR = "f[^G[";
        const string STR_SEND_PORT_ERROR = "|[gG[";

        const string STR_OPEN_PORT = "|[g{0}J܂D";
        const string STR_CLOSE_PORT = "|[g{0}܂D";

        const string STR_ERROR = "G[";
        const string STR_ERROR_CATEGORY_NOSELECT = "G[: CategoryIĂ܂D";
        const string STR_ERROR_CATEGORY_NOINCLUDE = "G[: Category:{0} ̓XgɊ܂܂Ă܂D";
        const string STR_ERROR_CATEGORY_EXISTPRODUCT = "G[: Category: {0}ɂ{1}Product܂܂܂DׂĂProductOɍ폜ĂD";
        const string STR_DELETE_CATEGORY = "Category:{0} 폜܂D";

        const string STR_ERROR_PRODUCT_NOSELECT  =  "G[: ProductIĂ܂D";
        const string STR_ERROR_PRODUCT_NOINCLUDE = "G[: Product:{0} ̓XgɊ܂܂Ă܂D";
        const string STR_ERROR_PRODUCT_NONE = "G[: Category:{0}ɑΉProduct݂͑܂D";
        const string STR_DELETE_PRODUCT = "Product:{0} 폜܂D";

        const string DEFAULT_ITEM_CATEGORY = "[Category]";
        const string DEFAULT_ITEM_PRODUCT = "[Product]";
        bool initialized = false;

        #endregion

        #region Constructor/Destructor

        public Panel()
        {
            InitializeComponent();
        }

        public Panel(MobiServer.TCPServer server, MobiServer.Message message)
            : base(Data.Base.Header, Data.Base.Header, server, message)
        {
            InitializeComponent();
          
            controlHolder = new Hashtable();
            productHolder = new Hashtable();
            LoadCategory();
            LoadProductAll();
            comboBoxCategory.SelectedIndex = 0;
            comboBoxProduct.SelectedIndex = 0;
            LoadSetting();
            if (checkBoxAutoStart.Checked)
            {
                OpenPort();
            }
            serverWriteHandler += new MobiServer.WriteHandler(serverControl.OnServerWriteEvent);
            serverControl.AddReadHandler(new MobiServer.ReadHandler(OnServerReadEvent));
            serverControl.messageHandler += new MobiServer.MessageHandler(messageControl.OnMessageEvent);
        
            initialized = true;
        }

        void ChangeGUIEnabled(bool flag)
        {
            checkBoxAutoPortScan.Enabled = flag;
            checkBoxAutoStart.Enabled = flag;
            comboBoxComPort.Enabled = flag;
        }

        private void Panel_Load(object sender, EventArgs e)
        {
            try
            {
                Hashtable h = IR.Control.Base.LoadIRControlAll();
                foreach (Base b in h.Values)
                {
                    StartControl(b);
                }
                initialized = true;
            }
            catch (Exception ex)
            {
                PrintError(ex.ToString());
            }
        }
        #endregion

        #region Server In/Out
        override public void OnServerReadEvent(object sender, MobiServer.ServerEventArgs e)
        {
            try
            {
                Data.Base b = Data.Base.Parse(e.Args);

                if (b != null)
                {
                    if (b.State == Data.States.Attach)
                    {
                        StartControl(new Base(b.Category, b.Product, GetPort()));
                    }
                }
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
            }
        }
        #endregion

        #region Serial Port and IR In/Out
        private void buttonOpen_Click(object sender, EventArgs e)
        {
            OpenPort();
        }

        private void buttonClose_Click(object sender, EventArgs e)
        {
            ClosePort();
        }

        void OpenPort()
        {
            buttonOpen.Enabled = false;
            ChangeGUIEnabled(false);
            try
            {
                if (!serialPort1.IsOpen)
                {
                    serialPort1.PortName = this.comboBoxComPort.SelectedItem.ToString();
                    //serialPort1.DataReceived += new System.IO.Ports.SerialDataReceivedEventHandler(OnDataReceived);
                    serialPort1.Open();
                }
                buttonClose.Enabled = true;
                PrintMessage(STR_OPEN_PORT, serialPort1.PortName);
            }
            catch(Exception ex)
            {
                PrintError(ex.ToString());
                buttonOpen.Enabled = true;
                buttonClose.Enabled = false;
                ChangeGUIEnabled(true);
            }
        }

        void ClosePort()
        {
            buttonClose.Enabled = false;
            try
            {
                if (serialPort1.IsOpen)
                {
                    serialPort1.Close();
                }

                PrintMessage(STR_CLOSE_PORT, serialPort1.PortName);
            }
            catch (Exception ex)
            {
                PrintError(ex.ToString());
            }

            buttonOpen.Enabled = true;
            ChangeGUIEnabled(true);
        }


        byte[] ReceiveIRData()
        {
            byte[] rxBytes = new byte[Data.Base.DATA_LENGTH + 2];

            try
            {
                if (serialPort1 != null && !serialPort1.IsOpen)
                {
                    OpenPort();
                }

                if (serialPort1 != null && serialPort1.IsOpen)
                {
                    byte b;
                    serialPort1.ReadTimeout = READ_TIMEOUT;
                    serialPort1.Write(new byte[] { Data.Commands.READ }, 0, 1);

                    //r  -> Y 
                    PrintMessage(STR_RECEIVE_START);
                    Application.DoEvents();
                    try
                    {
                        b = (byte)serialPort1.ReadByte();
                    }
                    catch (Exception ex)
                    {
                        PrintError(STR_RECEIVE_ERROR);
                        return null;
                    }

                    if (Data.Commands.ACK != b)
                    {
                        PrintError(STR_RECEIVE_ERROR);
                        return null;
                    }
                    PrintDebug(STR_RECEIVE_INIT);

                    //(Start(1) + Data(240) + End(1))
                    try
                    {
                        int length = rxBytes.Length;
                        int offset = 0;
                        while (offset < rxBytes.Length)
                        {
                            int l = serialPort1.Read(rxBytes, offset, length);
                            offset += l;
                            length -= l;
                        }
                        //serialPort1.Read(rxBytes, 0, rxBytes.Length);
                    }
                    catch (Exception ex)
                    {
                        PrintError(STR_RECEIVE_ERROR);
                        return null;
                    }

                    if (rxBytes[0] == Data.Commands.START_DATA &&
                        rxBytes[rxBytes.Length - 1] == Data.Commands.END_DATA)
                    {
                        PrintMessage(STR_RECEIVE_COMPLETE);

                        byte[] resultBytes = new byte[Data.Base.DATA_LENGTH];
                        Array.Copy(rxBytes, 1, resultBytes, 0, Data.Base.DATA_LENGTH);

                        return resultBytes;
                    }
                    else
                    {
                        PrintError(STR_RECEIVE_ERROR);
                        return null;
                    }
                }
                else
                {
                    PrintError(STR_OPEN_ERROR);
                    return null;
                }
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
                return null;
            }
        }

        bool SendIRData(byte[] data, int port)
        {
            try
            {
                if (data.Length != Data.Base.DATA_LENGTH)
                {
                    PrintError(STR_SEND_ERROR + String.Format("({0}:{1})",
                                                            STR_SEND_LENGTH_ERROR, data.Length));
                    return false;
                }
                //else if (port < Data.Commands.PORT_1 || port > Data.Commands.PORT_4)
                else if (port < 1 || port > 4)
                {
                    PrintError(STR_SEND_ERROR + String.Format("({0}:{1})",
                                                                STR_SEND_PORT_ERROR, data.Length));
                    return false;
                }

                if (serialPort1 != null && !serialPort1.IsOpen)
                {
                    OpenPort();
                }

                if (serialPort1 != null && serialPort1.IsOpen)
                {
                    byte b;
                    serialPort1.ReadTimeout = READ_TIMEOUT;

                    serialPort1.Write(new byte[] { Data.Commands.TRANSMIT }, 0, 1); //Transmit t  -> Receive Y 
                    //PrintDebug(STR_SEND_START);
                    try
                    {
                        b = (byte)serialPort1.ReadByte();
                    }
                    catch (Exception ex)
                    {
                        PrintError(STR_SEND_ERROR);
                        return false;
                    }
                    if (Data.Commands.ACK != b)
                    {
                        PrintError(STR_SEND_ERROR);
                        return false;
                    }

                    byte pbyte = Data.Commands.PORT_1;
                    switch (port)
                    {
                        case 1:
                            pbyte = Data.Commands.PORT_1;
                            break;
                        case 2:
                            pbyte = Data.Commands.PORT_2;
                            break;
                        case 3:
                            pbyte = Data.Commands.PORT_3;
                            break;
                        case 4:
                            pbyte = Data.Commands.PORT_4;
                            break;

                    }

                    serialPort1.Write(new byte[] { pbyte }, 0, 1); //Transmit 1-4 -> Receive Y 
                    //PrintDebug(STR_SEND_PORT);
                    try
                    {
                        b = (byte)serialPort1.ReadByte();
                    }
                    catch (Exception ex)
                    {
                        PrintError(STR_SEND_ERROR);
                        return false;
                    }

                    if (Data.Commands.ACK != b)
                    {
                        PrintError(STR_SEND_ERROR);
                        return false;
                    }
                   // PrintDebug(STR_SEND_INIT);

                    serialPort1.Write(data, 0, data.Length); //Transmit Data[240]
                    try
                    {
                        b = (byte)serialPort1.ReadByte();
                    }
                    catch (Exception ex)
                    {
                        PrintError(STR_SEND_ERROR);
                        return false;
                    }
                    if (Data.Commands.END_DATA != b)
                    {
                        PrintError(STR_SEND_ERROR);
                        return false;
                    }

                    PrintDebug(STR_SEND_COMPLETE);
                    return true;
                }
                else
                {
                    PrintError(STR_OPEN_ERROR);
                    return false;
                }
            }
            catch (Exception ex)
            {
                PrintError(ex.ToString());
                return false;
            }
        }

        void OnIRSendEvent(object sender, Data.IREventArgs e)
        {
            EnableControlAll(false);
            SendIRData(e.IRData.IRBytes, e.IRData.Port);
            EnableControlAll(true);
        }
        
        void OnIRReceiveEvent(object sender, Data.IREventArgs e)
        {
            EnableControlAll(false);
            Data.Base irData = e.IRData;
            irData.IRBytes = ReceiveIRData();
            if (irData.IRBytes != null)
            {
               foreach(Hashtable controlLine in controlHolder.Values){
                   foreach (Control.Base irControl in controlLine.Values)
                   {
                       if (irControl.SetData(irData))
                       {
                           break;
                       }
                   }
               }
            }
            EnableControlAll(true);
        }

        void EnableControlAll(bool enabled){
            foreach(Hashtable controlLine in controlHolder.Values){
                foreach (Control.Base irControl in controlLine.Values)
                {
                    irControl.SetEnabled(enabled);
                }
            }
        }

        bool TestIRData(string category, string product, int port)
        {

            string directory = Control.Base.DATA_DIR + @"\" +
                                String.Format("{0}_{1}", category, product);
            bool matched = false;

            if (!Directory.Exists(directory))
                return false;

            string[] files = Directory.GetFiles(directory);

            if (files.Length == 0)
                return false;

            foreach (string matchStr in TestCommands)
            {
                foreach (string file in files)
                {
                    string f = Path.GetFileNameWithoutExtension(file);
                    Match m = Regex.Match(f, String.Format("^{0}_(.+)", Control.Base.DATA_FILE_HEADER));
                    if (m.Success)
                    {
                        string command = m.Groups[1].ToString();
                        Match m2 = Regex.Match(command, matchStr, RegexOptions.IgnoreCase);
                        if (m2.Success)
                        {
                            byte[] data = null;
                            try
                            {
                                data = File.ReadAllBytes(file);
                                if (data.Length == Data.Base.DATA_LENGTH)
                                {
                                    Data.Base b = new Data.Base(Data.States.Out, category, product, command, port, data);
                                    EnableControlAll(false);
                                    SendIRData(b.IRBytes, b.Port);
                                    EnableControlAll(true);
                                    matched = true;
                                    PrintDebug("IRTest:{0}_{1}_{2}", category, product, command);
                                }
                            }
                            catch (Exception ex)
                            {
                            }
                            break;
                        }
                    }
                }
                if (matched)
                    break;
            }
            if (!matched)
            {
                PrintDebug("IRTest:{0}_{1}:Command not found.", category, product);

            }
            return matched;
        }

        #endregion

        #region Add/Delete Controls
        public void StartControl(Base control)
        {
            control.messageHandler += new MobiServer.MessageHandler(messageControl.OnMessageEvent);
            control.serverWriteHandler += new MobiServer.WriteHandler(serverControl.OnServerWriteEvent);
            serverControl.AddReadHandler(new MobiServer.ReadHandler(control.OnServerReadEvent));
            control.irReceiveHandler += new Data.IRReceiveHandler(OnIRReceiveEvent);
            control.irSendHandler += new Data.IRSendHandler(OnIRSendEvent);
            control.SetEnabled(true);
            control.Start();
            AddControl(control);
        }

        internal void WriteServerSockets(string[] args)
        {
            serverControl.WriteAllSockets(args);
        }

        public void AddControl(Base control)
        {
            Hashtable controlLine = null;
            bool addCompleted = false;
            
            //already exists?
            for (int i = 0; i < controlHolder.Count; i++)
            {
                controlLine = (Hashtable)controlHolder[i];
                for (int j = 0; j < controlLine.Count; j++)
                {
                    Base c = (Base)controlLine[j];
       
                    if(control.CheckAddress(c.Category, c.Product))
                    {
                        int num = this.Controls.IndexOf(c);
                        this.Controls[num].Enabled = true;
                        this.Refresh();
                        return;
                    }
                }
            }

            //insert a new control between existing controls. 
            for (int i = 0; i < controlHolder.Count; i++)
            {
                int lineWidth = 0;
                int y = 0;
                int lineHeight = 0;

                controlLine = (Hashtable)controlHolder[i];
                for (int j = 0; j < controlLine.Count; j++)
                {
                    Base c = (Base)controlLine[j];

                    if (!addCompleted)
                    {
                        lineWidth += c.Width;
                        y = c.Location.Y;
                        if (lineHeight < c.Height)
                            lineHeight = c.Height;
                    }
                    else
                    {
                        c.Location = new Point(c.Location.X, c.Location.Y + targetHeight);
                        controlLine[j] = c;
                    }
                }
                if (addCompleted)
                {
                    controlHolder[i] = controlLine;
                }
                else
                {
                    if (lineWidth + control.Width < this.Width)
                    {
                        control.Location = new Point(lineWidth, y);
                        targetControl = control;

                        if (lineHeight < control.Height)
                            targetHeight = control.Height - lineHeight;
                        else
                            targetHeight = 0;

                        this.Invoke(new EventHandler(AddControl));
                        controlLine.Add(controlLine.Count, control);
                        controlHolder[i] = controlLine;
                        addCompleted = true;
                    }
                }
            }
            //add a new control
            if (!addCompleted)
            {
                control.Location = new Point(0, this.Height);// - 30
                targetControl = control;
                targetHeight = targetControl.Height;
                this.Invoke(new EventHandler(AddControl)); 
                controlLine = new Hashtable();
                controlLine.Add(controlLine.Count, control);
                controlHolder.Add(controlHolder.Count, controlLine);
            }

            return;
        }

        void AddControl(object sender, EventArgs e)
        {
            this.SuspendLayout();
            this.Controls.Add(targetControl);
            if (CheckScreenHeight(this.Height + targetHeight + this.Location.Y))
                this.Height += targetHeight;
            else
                this.Height = System.Windows.Forms.Screen.PrimaryScreen.WorkingArea.Height - this.Location.Y;

            this.ResumeLayout(true);
        }

        bool CheckScreenHeight(int height)
        {
            if (height <= System.Windows.Forms.Screen.PrimaryScreen.WorkingArea.Height)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        private void buttonAdd_Click(object sender, EventArgs e)
        {
            try
            {
                string category = comboBoxCategory.SelectedItem.ToString();
                string product = comboBoxProduct.SelectedItem.ToString();
                StartControl(new Base(category, product, GetPort()));

                Data.Base b = new Data.Base(Data.States.Attach, category, product);
                string[] args = b.ToArray();
                MobiServer.ServerEventArgs se = new MobiServer.ServerEventArgs(args);//Data
                serverWriteHandler.Invoke(this, se);

            }
            catch (Exception ex)
            {
                PrintError(ex.ToString());
            }
        }

        #endregion

        #region Save & Load

        public void SaveControls()
        {
            foreach (Hashtable controlLine in controlHolder.Values)
            {
                foreach (Base control in controlLine.Values)
                {
                    control.SaveSetting();
                }
            }
        }

        public void SaveCategory()
        {
            StreamWriter fileStream = null;
            string file = System.Windows.Forms.Application.StartupPath + @"\" + SETTING_DIR + @"\" + SETTING_FILE_CATEGORY;
            try
            {
                fileStream = new StreamWriter(file, false);
                foreach (string item in comboBoxCategory.Items)
                {
                    if (item == DEFAULT_ITEM_CATEGORY)
                        continue;
                    fileStream.WriteLine(item);
                }

            }
            catch (Exception ex)
            {
                PrintError(ex.ToString());
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        public void LoadCategory()
        {
            StreamReader fileStream = null;
            string file = System.Windows.Forms.Application.StartupPath + @"\" + SETTING_DIR + @"\" + SETTING_FILE_CATEGORY;


            if (!File.Exists(file))
            {
                return;
            }
           
            try
            {
                comboBoxCategory.Items.Clear();
                comboBoxCategory.Items.Add(DEFAULT_ITEM_CATEGORY);

                fileStream = new StreamReader(file);
                while (true)
                {
                    string line = fileStream.ReadLine();

                    if (line == null)
                        break;

                    if (line == "")
                        continue;

                    if (comboBoxCategory.Items.Contains(line))
                        continue;

                    comboBoxCategory.Items.Add(line);
                }
            }
            catch (Exception ex)
            {
                PrintError(ex.ToString());
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        public void SaveProduct(string category)
        {
            StreamWriter fileStream = null;
            string file = String.Format(@"{0}\{1}\{2}_{3}.txt",
                                        System.Windows.Forms.Application.StartupPath,
                                        SETTING_DIR, SETTING_FILE_PRODUCT_HEADER, category);
            
            if (!productHolder.Contains(category))
                return;

            try
            {
                string [] array = (string [])productHolder[category];

                fileStream = new StreamWriter(file, false);

                foreach (string item in array)
                {
                    if (item == DEFAULT_ITEM_PRODUCT)
                        continue;

                    fileStream.WriteLine(item);
                }

            }
            catch (Exception ex)
            {
                PrintError(ex.ToString());
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        public void LoadProduct(string category)
        {
            StreamReader fileStream = null;
            string file = String.Format(@"{0}\{1}\{2}_{3}.txt",
                                        System.Windows.Forms.Application.StartupPath,
                                        SETTING_DIR, SETTING_FILE_PRODUCT_HEADER, category);
            
            if (!File.Exists(file))
            {
                return;
            }

            if (productHolder.Contains(category))
            {
                productHolder.Add(category, null);//new ComboBox.ObjectCollection(comboBoxProduct)
            }

            try
            {
                fileStream = new StreamReader(file);
                ArrayList array = new ArrayList(); 
                while (true)
                {
                    string line = fileStream.ReadLine();

                    if (line == null)
                        break;

                    if (line == "")
                        continue;

                    if (line == DEFAULT_ITEM_PRODUCT)
                        continue;

                    if (array.Contains(line))
                        continue;

                    array.Add(line);
                }
                string[] items = new string[array.Count];
                array.CopyTo(items);
                productHolder[category] = items;
            }
            catch (Exception ex)
            {
                PrintError(ex.ToString());
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        public void LoadProductAll()
        {
            foreach (string item in comboBoxCategory.Items)
            {
                LoadProduct(item);
            }
            if (comboBoxCategory.SelectedItem != null)
            {
                string category = comboBoxCategory.SelectedItem.ToString();
                ChangeProduct(category);
            }
        }

        public void SaveSetting()
        {
            StreamWriter fileStream = null;
            string file = System.Windows.Forms.Application.StartupPath + @"\" + SETTING_DIR + @"\" + SETTING_FILE;
            try
            {
                fileStream = new StreamWriter(file, false);
                fileStream.WriteLine("{0}\t{1}\t{2}\t{3}", comboBoxComPort.SelectedItem.ToString(),
                    checkBoxAutoStart.Checked, checkBoxAutoPortScan.Checked,checkBoxIRTest.Checked);
              
                fileStream.WriteLine("{0}\t{1}\t{2}", comboBoxCategory.SelectedItem.ToString(), comboBoxProduct.SelectedItem.ToString(),comboBoxPort.SelectedItem.ToString());
            }
            catch (Exception ex)
            {
                PrintError(ex.ToString());
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        public override void LoadSetting()
        {
            StreamReader fileStream = null;
            string file = System.Windows.Forms.Application.StartupPath + @"\" + SETTING_DIR + @"\" + SETTING_FILE;
            string line = "";
            string[] args = null;

            try
            {
                if (!File.Exists(file))
                    return;

                fileStream = new StreamReader(file);

                line = fileStream.ReadLine();
                args = line.Split("\t".ToCharArray());

                comboBoxComPort.Text = args[0];
                checkBoxAutoStart.Checked = bool.Parse(args[1]);
                checkBoxAutoPortScan.Checked = bool.Parse(args[2]);
                checkBoxIRTest.Checked = bool.Parse(args[3]);

                line = fileStream.ReadLine();
                args = line.Split("\t".ToCharArray());
                if (comboBoxCategory.Items.Contains(args[0]))
                {
                    comboBoxCategory.SelectedItem = args[0];
                }
                if (comboBoxProduct.Items.Contains(args[1]))
                {
                    comboBoxProduct.SelectedItem = args[1];
                }
                if (comboBoxPort.Items.Contains(args[2]))
                {
                    comboBoxPort.SelectedItem = args[2];
                }
            }
            catch (Exception ex)
            {
                PrintError(ex.ToString());
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        #endregion

        #region Category/Product/Port

        private void comboBoxCategory_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case System.Windows.Forms.Keys.Enter:
                    AddCategory(comboBoxCategory.Text);
                    break;
                case System.Windows.Forms.Keys.Delete:
                    DeleteCategory(comboBoxCategory.Text);
                    break;
            }
        }

        bool AddCategory(string text)
        {
            if (text != "" && !comboBoxCategory.Items.Contains(text))
            {
                comboBoxCategory.Items.Add(text);
                comboBoxCategory.SelectedItem = text;
                SaveCategory();
                return true;
            }
            else
            {
                return false;
            }
        }

        bool DeleteCategory(string text)
        {
            if (text == DEFAULT_ITEM_CATEGORY)
            {
                return false;
            }
            else if (text == "")
            {
                System.Windows.Forms.MessageBox.Show(STR_ERROR_CATEGORY_NOSELECT, STR_ERROR);
                return false;
            }
           
            else if (!comboBoxCategory.Items.Contains(text))
            {
                System.Windows.Forms.MessageBox.Show(String.Format(STR_ERROR_CATEGORY_NOINCLUDE, text),  STR_ERROR);
                return false;
            }
            else if (productHolder.Contains(text) && ((string [])productHolder[text]).Length > 0)
            {
                System.Windows.Forms.MessageBox.Show(
                            String.Format(STR_ERROR_CATEGORY_EXISTPRODUCT,
                                               text, ((string [])productHolder[text]).Length), STR_ERROR);
                return false;
            }
            else
            {
                comboBoxCategory.Items.Remove(text);
                comboBoxCategory.SelectedIndex = 0;
                PrintMessage(STR_DELETE_CATEGORY, text);
                SaveCategory();
                return true;
            }
        }

        private void comboBoxProduct_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case System.Windows.Forms.Keys.Enter:
                    AddProduct(comboBoxProduct.Text);
                    break;
                case System.Windows.Forms.Keys.Delete:
                    DeleteProduct(comboBoxCategory.Text);
                    break;
            }
        }

        bool ChangeProduct(string category)
        {
            if (productHolder.Contains(category))
            {
                comboBoxProduct.Items.Clear();
                comboBoxProduct.Items.Add(DEFAULT_ITEM_PRODUCT);
                string [] array = (string [])productHolder[category];
                foreach (string item in array)
                {
                    comboBoxProduct.Items.Add(item);
                }
                
                comboBoxProduct.SelectedIndex = 0;
                
                return true;
            }
            else
            {
                comboBoxProduct.Items.Clear();
                comboBoxProduct.Items.Add(DEFAULT_ITEM_PRODUCT);
                comboBoxProduct.SelectedIndex = 0;
                return false;
            }
        }

        bool AddProduct(string text)
        {
            if (text != "" && !comboBoxProduct.Items.Contains(text))
            {
                string category = comboBoxCategory.SelectedItem.ToString();
                if (category == DEFAULT_ITEM_CATEGORY)
                {
                    System.Windows.Forms.MessageBox.Show(STR_ERROR_CATEGORY_NOSELECT, STR_ERROR);
                    return false;
                }
                else{
                    comboBoxProduct.Items.Add(text);
                    comboBoxProduct.SelectedItem = text;

                     string [] array = new string[comboBoxProduct.Items.Count];
                    comboBoxProduct.Items.CopyTo(array, 0);
                    if (productHolder.Contains(category))
                    {
                        productHolder[category] = array;
                    }
                    else
                    {
                        productHolder.Add(category, array);
                    }
                    SaveProduct(category);
                    return true;
                }
            }
            else
            {
                return false;
            }
        }

        bool DeleteProduct(string text)
        {
            string category = comboBoxCategory.SelectedItem.ToString();
            if (text == comboBoxProduct.Items[0].ToString())
            {
                return false;
            }
            else if (text == "")
            {
                System.Windows.Forms.MessageBox.Show(STR_ERROR_PRODUCT_NOSELECT, STR_ERROR);
                return false;
            }
            else if (!comboBoxProduct.Items.Contains(text))
            {
                System.Windows.Forms.MessageBox.Show(String.Format(STR_ERROR_PRODUCT_NOINCLUDE, text), STR_ERROR);
                return false;
            }
            else if (!productHolder.Contains(category))
            {
                System.Windows.Forms.MessageBox.Show(String.Format(STR_ERROR_PRODUCT_NONE, category), STR_ERROR);
                return false;
            }
            else
            {
                comboBoxProduct.Items.Remove(text);
                comboBoxProduct.SelectedIndex = 0;
                productHolder[category] = comboBoxProduct.Items;
                PrintMessage(STR_DELETE_PRODUCT, text);
                SaveProduct(category);
                return true;
            }
        }

        private void comboBoxCategory_SelectedIndexChanged(object sender, EventArgs e)
        {
            string category = comboBoxCategory.SelectedItem.ToString();
            ChangeProduct(category);
        }

        private void comboBoxProduct_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (initialized && checkBoxIRTest.Checked)
            {
                TestIRData(comboBoxCategory.SelectedItem.ToString(), 
                                comboBoxProduct.SelectedItem.ToString(), GetPort());
            }
        }

        private void comboBoxPort_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (initialized && checkBoxIRTest.Checked)
            {
                TestIRData(comboBoxCategory.SelectedItem.ToString(), 
                                comboBoxProduct.SelectedItem.ToString(), GetPort());
            }
        }

        int GetPort()
        {
            int port = comboBoxPort.SelectedIndex;
            if (port == 0 || port > 4)
                port = 1;

            return port;
        }

        #endregion
    }
}

